#include "stream.h"
#include <stdio.h>
#include <stdlib.h>

FILE* file=NULL;

BitStream::BitStream():Buffer(0), BufferLength(0), ByteStuffing(false)
{ Name[0]=0; }

BitStream::~BitStream()
{ if( file!=NULL ) Close(); }

bool BitStream::Open(const char* name, const char* attr)
{ file=fopen(name, attr);

  if( file==NULL ) return false;

  int i=0;
  do{ Name[i]=name[i];
      i=i+1;
      if( i==32 ){ Name[31]=0; break; }
    }while( name[i]!=0 );

  return true;
}

bool BitStream::Close()
{ if( file!=NULL )
    {
      if( BufferLength!=0 ) fputc(Buffer, file);

      Buffer=0;
      BufferLength=0;

      fclose(file);
      file=NULL;
      return true;
    }
  return false;
}

bool BitStream::IsEndOfStream()
{ if( file==NULL ) return true;

  if( BufferLength!=0 ) return false;

  if( feof(file)==0 ) return false;

  return true;
}

/*----------------------------------------------------------------------------*/

bool BitStream::Put(Byte val)
{ if( file==NULL ) return false;

  if( BufferLength>=8 )
    { fputc(Buffer,file);
      if( ByteStuffing==true && Buffer==(Byte)0xFF )
          fputc(0x00,file);
      Buffer=0;
      BufferLength=0;
    }

  Byte mask=(0x80>>BufferLength);

  switch( val )
    { case 0: mask=~mask; Buffer&=mask; break;
      case 1:             Buffer|=mask; break;
      default: break;
    }

  BufferLength+=1;

  return true;
}

bool BitStream::Get(Byte* val)
{ if( file==NULL ) return false;

  if( BufferLength==0 )
    {
      Buffer=fgetc(file);

      if( feof(file)!=0 ){ Buffer=0; BufferLength=0; return false; }

      if( ByteStuffing==true && Buffer==(Byte)0xFF )
          fgetc(file); /* h stuffing 0x00 */

      BufferLength=8;
    }

  (*val) = (Buffer & 0x80) >> 7;

  Buffer = Buffer<<1;

  BufferLength-=1;

  return true;
}

/*----------------------------------------------------------------------------*/

bool BitStream::PutStream(DWord code, Byte length)
{ if( file==NULL ) return false;

  if( (8*sizeof(DWord))<length ) return false;

  DWord mask= 1<<(length-1);

  for(int i=0; i<length; i++)
     {
       if( ( code&(mask>>i) )!=0 )
         { Put(1); }
       else
         { Put(0); }
     }

  return true;
}

DWord BitStream::StreamIn(Byte length)
{ if( file==NULL ) return 0;

  if( (8*sizeof(DWord))<length ) return 0;

  Byte  i, val;
  DWord bits=0;

  for(i=1; i<=length; i++)
     { bits = bits << 1;
       if( Get(&val)==false ) return 0;
       bits = bits | val;
     }

  return bits;
}

bool BitStream::GetStream(DWord* code, Byte length)
{ if( file==NULL ) return false;

  if( (8*sizeof(DWord))<length ) return false;

  Byte val;

  for(int i=1; i<=length; i++)
     { if( Get(&val)==false ) return false;

       (*code)=(*code)<<1;

       if( val!=0 ) (*code) = (*code) | 0x01 ;
     }

  return true;
}

/*----------------------------------------------------------------------------*/

void BitStream::Align(bool flag)
{ if( (BufferLength%8)==0 ) return;

  int i, j=8-BufferLength%8;

  for(int i=1; i<=j; i++ )
     {
       if( flag )
         { Put(1); }
       else
         { Put(0); }
     }
}

/*----------------------------------------------------------------------------*/

BitStream& BitStream::operator<<( const char*  str   )
{ int i=0;

  do{ switch( str[i] )
        { case '0': Put(0); break;
          case '1': Put(1); break;
          default : break;
        }
      i++;
    }while( str[i]!=0 );

  return (*this);
}


BitStream& BitStream::operator<<( const DWord& dword  )
{Byte  i;
 DWord mask=0x80000000;

 for(i=0; i<8*(sizeof(DWord)); i++)
    {
      if( ( dword&(mask>>i) )!=0 )
        { Put(1); }
      else
        { Put(0); }
    }

 return (*this);
}
