#ifndef __type__
#define __type__

typedef unsigned char  Byte ; /* 8-bit  */
typedef unsigned short Word ; /* 16-bit */
typedef unsigned long  DWord; /* 32-bit */

enum BlockType
{ Y_Block =0,
  CbBlock,
  CrBlock,
  ForbiddenBlock
};

enum PictureType
{ Forbidden=0,
  I_Picture=1,
  P_Picture=2,
  B_Picture=3,
  D_Picture=4
};/* 3-bits coding */


typedef struct MacroBlock_Type
{ bool Intra;
  bool pattern;
  bool Backward;
  bool Forward;
  bool Quant;
  Byte pattern_code;
} MacroType;

/*
  MD  --> motion displance
  dMD = motion_code * f - r
*/
typedef struct Displacement
{ short Displace; /* motion_code -16...+16 */
  Byte  Residual;
}Vector;


class Motion
{ public:
  Vector Horizontal;
  Vector Vertical  ;

  bool  Full_Pixel;
  Byte  Factor_Code; /* f_code 1...7 */

  Byte  r_size(){ return (Factor_Code-1     ); }
  short factor(){ return (1<<(Factor_Code-1)); }

  Motion():Full_Pixel(true),Factor_Code(1){};
};


typedef struct Time_Code
{ bool Drop_Frame_Flag; // 1 is 29.97Hz else 0 //
  Byte Hours;    // 5 bits //
  Byte Minutes;  // 6 bits //
  Byte Seconds;  // 6 bits //
  Byte Pictures; // 6 bits //
}TimeCode;

/*----------------------------------------------------------------------------*/
#define Block_Width  8
#define Block_Height 8
#define MCU_Max      10

struct Component
{ Byte Horizontal;//Vˤ
  Byte Vertical;  //Vˤ
};

struct Block
{ BlockType Type;
  int       Value[Block_Height][Block_Width];
};

class Picture
{ private:
    Byte  Component_H_Max, Component_V_Max;

    DWord Block_Address;

    void _copy_to_MCU(Word x, Word y, int id);
    void _copy_to_Pixel(Word x, Word y, int id);

    void _find_HV_max();

  protected:

    void _set_HV(int index, Byte HV=0x11);

    void _clear_picture();
    void _clear_block();
    void _create_array(Word w, Word h);
    void _delete_array();

    bool _get_min_code_unit(int address); /* Ǧ^ false --> ܵFΨSm */
    bool _put_min_code_unit(int address); /* Ǧ^ false --> ܵFΨSm */

    Component C[4]; //C[0] 

  public:
    Block   MCU[10]; //minum code unit

    DWord** Pixel;
    Word    Width, Height;

    Picture& operator[] (Word address);

    //Picture& operator() (Word x, Word y);

    void Create(Word w, Word h);

    void Release();

    inline bool IsEmpty(){ if(Pixel==0)return true; return false; };

    Picture();
   ~Picture();
};

/*----------------------------------------------------------------------------*/

DWord RGB2YCbCr(DWord RGB  );
DWord YCbCr2RGB(DWord YCbCr);

typedef struct BMP_Base
{ Word  Identify;

  DWord File_Size;
  DWord Reseseve;
  DWord Data_Offset;
  DWord Header_Size;
  DWord Width;
  DWord Height;

  Word  Planes;
  Word  Bit_Per_Pixel;

  DWord Compression;
  DWord Bitmap_Data_Size;
  DWord HResolution;
  DWord VResolution;
  DWord Colors;
  DWord Importat_Colors;

  BMP_Base(): Identify(0x4D42),
              File_Size(0),
              Reseseve(0),
              Data_Offset(54),
              Header_Size(0x28),
              Width(0),
              Height(0),
              Planes(1),
              Bit_Per_Pixel(24),
              Compression(0),
              Bitmap_Data_Size(0x300),
              HResolution(0xEC4),
              VResolution(0xEC4),
              Colors(0),
              Importat_Colors(0)
              {};
} BMP;

extern bool ReadBMP (const char* file, Picture* pic);
extern bool WriteBMP(const char* file, Picture* pic);

/*----------------------------------------------------------------------------*\
  ϥ type.h Oo]t  picture.cpp , bmp.cpp


\*----------------------------------------------------------------------------*/

#endif

